.macpack atari

.if (Atari=5200)
  .include "atari5200.inc"
  .segment "RUN"
    scrcode "       BLOWSUB       "
    .byte $FF
    .word main
  .segment "ROM"
  Atari5200 = 1
  SKCTL_VALUE = $02
  ROM_CHSET   = $F8
.else
  .include "atari.inc"
  .segment "HEAD"
    .byte $FF,$FF
    .word code_start
    .word RAM_IMAGE+RAM_END-RAM_START-1
  .segment "INIT"
    .word RUNAD
    .word RUNAD+1
    .word main
  .segment "CODE"
  SKCTL_VALUE = $03
  ROM_CHSET   = $E0
.endif

code_start:

NUM_SUBS     = 9
NUM_CHARGES  = 4
NUM_MISSILES = 4

PM_MEMORY      = $3800
MISSILE_MEMORY = PM_MEMORY+$300
PLAYER0_MEMORY = PM_MEMORY+$400
PLAYER1_MEMORY = PM_MEMORY+$500
PLAYER2_MEMORY = PM_MEMORY+$600
PLAYER3_MEMORY = PM_MEMORY+$700

TITLE_SCREEN = 0
GAME_ACTIVE  = 1

CH_START = $0C
CH_ONE   = $0F
CH_TWO   = $0E
CH_THREE = $0D
CH_PAUSE = $08
CH_RESET = $04

NUM_POT_READINGS = 4

CALIBRATE_TRAKBALL_REQUEST = 4

zp_source    = $80
zp_dest      = $82
zp_length    = $84
zp_dli_count = $86
zp_vbi_flag  = $87
zp_offset    = $88
zp_temp      = $89
zp_pot2      = $8A
zp_ball_cx   = $8B
zp_trig0     = $8C
zp_pot_index = $8D
zp_calibrate = $8E
ship_control = $8F
high_score   = ship_control+1
zp_consol    = high_score+4

charset1:
  .incbin "charsets.dat"

charset2 = charset1+$400
charset3 = charset2+$400
charset4 = charset3+$400
charset5 = charset4+$400
charset6 = charset5+$400
charset7 = charset6+$400
charset8 = charset7+$400

digits_charset:
  .incbin "digits_charset.dat"

ship_colours:
  .byte $02,$04,$0A
  
rowboat_colours:
  .byte $36,$F2,$F4
  
sub_colours:
  .byte $0C,$04,$08

eel_colours:
  .byte $B4,$B0,$BC

mine_colours:
  .byte $0E,$02,$08

text_colours:
  .byte $02,$08,$0C

; *****************************************************************************
; * Main program                                                              *
; *****************************************************************************
.proc main
  lda #$01
  sta COLDST

  lda #DIGITAL_CONTROL
  sta ship_control
  
  lda #$00
  sta high_score
  sta high_score+1
  sta high_score+2

reset:
  sei
  lda #$00
  sta NMIEN
  sta IRQEN
  sta DMACTL
  sei

  ldx #$FF
  txs
  cld

  jsr reset_hardware

  lda #SKCTL_VALUE
  sta SKCTL

  lda #<RAM_START
  sta zp_dest
  lda #>RAM_START
  sta zp_dest+1
  lda #<RAM_IMAGE
  sta zp_source
  lda #>RAM_IMAGE
  sta zp_source+1
  lda #<(RAM_END-RAM_START)
  sta zp_length
  lda #>(RAM_END-RAM_START)
  sta zp_length+1
  jsr memory_copy
  jsr memory_clear
  
.ifdef Atari5200
  jsr init_trak_ball
  jsr select_keypad

  lda KBCODE
  sta kbcode
  lda #$00
  sta debounce_count
.else
  lda CONSOL
  sta zp_consol
.endif

  lda #$FF
  sta CH

  lda #<empty_display_list
  sta DLISTL
  lda #>empty_display_list
  sta DLISTH

  lda #<vbi
  sta VVBLKI
  lda #>vbi
  sta VVBLKI+1

  lda #<dli
  sta VDSLST
  lda #>dli
  sta VDSLST+1

  lda #>PM_MEMORY
  sta PMBASE
  
  lda #$03
  sta GRACTL
  
  lda #$11
  sta PRIOR
    
  lda #$3E
  sta DMACTL
  
  lda #TITLE_SCREEN
  sta game_state

  jsr init_audio
  jsr init_lines
  jsr init_charges
  jsr init_missiles
  jsr init_subs
  jsr init_ship
  jsr init_score

:
  lda VCOUNT
  bne :-

  lda #$C0
  sta NMIEN

  lda #$00
  sta zp_vbi_flag
  
vb_wait:
  lda zp_vbi_flag
  beq vb_wait

  jsr check_keypad
  
  jsr process_ship
  jsr process_subs
  jsr check_collisions
  jsr process_charges
  jsr process_missiles
  jsr display_status
  jsr process_audio

  lda #$00
  sta zp_vbi_flag

  jmp vb_wait
.endproc

; *****************************************************************************
; * Reset hardware                                                            *
; *****************************************************************************
.proc reset_hardware
  lda #$00
  ldx #$1F
:
  sta HPOSP0,x
  sta AUDF1,x
  sta DMACTL,x
  dex
  bpl :-
  rts
.endproc

; *****************************************************************************
; * Vertical Blank Interrupt                                                  *
; *****************************************************************************
.proc vbi
.ifdef Atari5200
  pha
  txa
  pha
  tya
  pha
.endif

  lda #$00
  sta COLPF1
  sta zp_dli_count
  lda dli_colbk
  sta COLPF2
  sta COLBK
  lda #ROM_CHSET
  sta CHBASE

  lda #<display_list
  sta DLISTL
  lda #>display_list
  sta DLISTH

  jsr draw_ship
  jsr draw_subs
  jsr draw_charges
  jsr draw_missiles
  jsr adjust_colours

.ifdef Atari5200
  lda zp_calibrate
  bne calibrate
.endif

  jsr read_joystick
  jsr read_keypad
  
exit_vbi:
  lda #$01
  sta zp_vbi_flag

.ifdef Atari5200
  pla
  tay
  pla
  tax
  pla
  rti

calibrate:
  jsr calibrate_trakball
  jmp exit_vbi
.else
  jmp XITVBV
.endif

.endproc

; *****************************************************************************
; * Display List Interrupt                                                    *
; *****************************************************************************
.proc dli
  pha

  sta WSYNC

  txa
  pha

  ldx zp_dli_count

  lda dli_colbk,x
  sta COLBK
 
  lda hscrol_values,x
  sta HSCROL
  
  lda dli_charsets,x
  sta CHBASE

  lda dli_colpf0,x
  sta COLPF0
  lda dli_colpf1,x
  sta COLPF1
  lda dli_colpf2,x
  sta COLPF2

  inc zp_dli_count

  pla
  tax
  pla
  rti
.endproc

; *****************************************************************************
; * Check keys                                                                *
; *****************************************************************************
.proc check_keypad
  lda CH
  cmp #CH_RESET
  beq reset

  lda game_state
  cmp #GAME_ACTIVE
  beq :+
  jmp check_title_screen_keys
:
  jmp check_in_game_keys
  
reset:
  jmp main::reset
.endproc

; *****************************************************************************
; * Check title screen keys                                                   *
; *****************************************************************************
.proc check_title_screen_keys
  lda CH
  cmp #CH_ONE
  beq one
  cmp #CH_TWO
  beq two
.ifdef Atari5200
  cmp #CH_THREE
  beq three
.endif
  cmp #CH_START
  beq start
  rts

one:
  lda #DIGITAL_CONTROL
  sta ship_control
.ifdef Atari5200
  jsr select_keypad
.endif
  ldy #$08
  jmp set_controller_text

two:
  lda #ANALOG_CONTROL
  sta ship_control
.ifdef Atari5200
  jsr select_keypad
.endif
  ldy #$08
  jmp set_controller_text

.ifdef Atari5200
three:
  lda zp_ball_cx
  cmp #$C0
  bcc trakball_detected
  rts

trakball_detected:
  lda #TRAKBALL_CONTROL
  sta ship_control
  jsr select_keypad
  ldy #$08
  jmp set_controller_text
.endif

start:
  lda #GAME_ACTIVE
  sta game_state
  
  jsr init_charges
  jsr init_missiles
  jsr init_subs
  jsr init_ship
  jsr init_score
  rts
.endproc

; *****************************************************************************
; * Check in-game keys                                                        *
; *****************************************************************************
.proc check_in_game_keys
  lda CH
  cmp #CH_PAUSE
  beq pause
  rts

pause:
  lda #$00
  sta AUDC1
  sta AUDC2
  sta AUDC3
  sta AUDC4
  
  lda #$FF
  sta CH
  
wait:
  lda CH
  cmp #CH_PAUSE
  bne wait
  rts
.endproc

; *****************************************************************************
; * Adjust screen colours                                                     *
; *****************************************************************************
.proc adjust_colours
  lda game_state
  cmp #GAME_ACTIVE
  beq check_mine
  jmp title_screen_colours

check_mine:
  lda mine_detonated
  beq check_ship
  jsr mine_screen_colours  
  
check_ship:
  lda ship_type
  cmp #SHIP_EXPLODING
  beq exploding
  cmp #SHIP_EXPLODING_ELECTRIC
  beq exploding
  rts

exploding:
  jmp explode_screen_colours
.endproc

; *****************************************************************************
; * Make colours all blue for title screen                                    *
; *****************************************************************************
.proc title_screen_colours
  lda #$8C
  sta COLBK
  sta COLPF2
  sta dli_colbk
  
  ldx #$00
loop:
  cpx #$03
  bne make_blue

  lda sub_type+1
  cmp #SUB_TEXT_LEFT
  bcs next
  
make_blue:
  lda dli_colpf0,x
  and #$0F
  ora #$80
  sta dli_colpf0,x
  lda dli_colpf1,x
  and #$0F
  ora #$80
  sta dli_colpf1,x
  lda dli_colpf2,x
  and #$0F
  ora #$80
  sta dli_colpf2,x
  lda dli_colbk,x
  and #$0F
  ora #$80
  sta dli_colbk,x
next:
  inx
  cpx #(NUM_SUBS+2)
  bne loop
  rts
.endproc

; *****************************************************************************
; * Flash red for exploding mine                                              *
; *****************************************************************************
.proc mine_screen_colours
  dec mine_detonated
  ldx mine_detonated
  ldy #$01
loop:
  lda dli_colbk,y
  and #$0F
  ora hue,x
  sta dli_colbk,y
next:
  iny
  cpy #(NUM_SUBS+2)
  bne loop
  rts
  
hue:
  .byte $80,$30,$30,$30,$80,$30,$30,$30,$80,$30,$30,$30,$80,$30,$30,$30
.endproc

; *****************************************************************************
; * Flash top of screen for exploding boat                                    *
; *****************************************************************************
.proc explode_screen_colours
  ldx ship_frame
  lda colours,x
  sta COLBK
  sta COLPF2
  sta dli_colbk
  rts
  
colours:
  .byte $00,$00,$0E,$0E,$00,$00,$0E,$0E
  .byte $00,$00,$BE,$BE,$00,$00,$BE,$BE
  .byte $00,$00,$3E,$3E,$00,$00,$3E,$3E
  .byte $00,$00,$0E,$0E,$00,$00,$0E,$8C
.endproc

; *****************************************************************************
; * Memory copy                                                               *
; *****************************************************************************
.proc memory_copy
  ldx #$00

loop:
  lda zp_length
  bne :+
  lda zp_length+1
  beq done
  dec zp_length+1
:
  dec zp_length
  
  lda (zp_source,x)
  sta (zp_dest,x)
  
  inc zp_source
  bne skip1
  inc zp_source+1
skip1:

  inc zp_dest
  bne skip2
  inc zp_dest+1
skip2:

  jmp loop
  
done:
  rts
.endproc

; *****************************************************************************
; * Clear RAM from zp_dest/+1 to $4000                                        *
; *****************************************************************************
.proc memory_clear
  lda #$00
  ldy #$00
loop:
  sta (zp_dest),y
  inc zp_dest
  bne loop
  inc zp_dest+1
  ldx zp_dest+1
  cpx #$40
  bne loop
  rts
.endproc

; *****************************************************************************
; * Read joystick                                                             *
; *****************************************************************************
.proc read_joystick
.ifdef Atari5200
  lda ship_control
  cmp #TRAKBALL_CONTROL
  beq read_trakball_data
  
  lda TRIG0
  sta zp_trig0
  
  inc zp_pot_index
  lda zp_pot_index
  and #(NUM_POT_READINGS-1)
  tax

  lda POT0
  sta pot0_readings,x
  
  cmp #114+50
  rol STICK0
  cmp #114-50
  rol STICK0
  lda POT1
  cmp #114+50
  rol STICK0
  cmp #114-50
  rol STICK0
  lda STICK0
  eor #2+8
  and #$0F
  sta STICK0

  sta POTGO
  rts

read_trakball_data:
  lda TRIG1
  sta zp_trig0

  lda POT2
  sta zp_pot2

  sta POTGO
  rts
.else
  lda ship_control
  cmp #ANALOG_CONTROL
  beq paddles
  
joystick:
  lda PORTA
  and #$0F
  sta STICK0
  lda TRIG0
  sta zp_trig0
  rts

paddles:
  inc zp_pot_index
  lda zp_pot_index
  and #(NUM_POT_READINGS-1)
  tax

  lda POT2
  sta pot0_readings,x

  lda PORTA
  lsr
  lsr
  lsr
  lsr
  lsr
  lsr
  and #$01
  sta zp_trig0
  
  sta POTGO
  rts  
.endif
.endproc

; *****************************************************************************
; * Read keypad                                                               *
; *****************************************************************************
.proc read_keypad
  lda #$FF
  sta CH

.ifdef Atari5200
  lda SKSTAT
  lsr
  lsr
  lsr
  and zp_trig0
  sta zp_trig0

  lda KBCODE
  cmp kbcode
  bne key_pressed
  
  lda debounce_count
  bne debounce
  rts

key_pressed:
  sta kbcode
  lda #$06
  sta debounce_count
  rts

debounce:
  dec debounce_count
  beq report_key
  rts
  
report_key:
  lda kbcode
  lsr
  and #$0F
  sta CH
  rts
.else
  ; key held?
  lda SKSTAT
  and #$04
  bne read_consol

  ; if escape then reset
  lda KBCODE
  cmp #$1C
  bne read_consol
  lda #CH_RESET
  sta CH
  rts

read_consol:
  lda CONSOL
  and #$07
  cmp zp_consol
  bne change
  rts
    
change:
  cmp #$07
  beq released
  sta zp_consol
  rts
  
released:
  lda zp_consol
  cmp #$06
  beq start
  cmp #$05
  beq select
  cmp #$03
  beq option

done:
  lda #$07
  sta zp_consol
  rts
  
start:
  lda #CH_START
  sta CH
  jmp done

select:
  ldx ship_control
  lda toggle_ship_control_values,x
  sta CH
  jmp done
  
toggle_ship_control_values:
  .byte CH_TWO,CH_ONE

option:
  lda #CH_PAUSE
  sta CH
  jmp done
.endif
.endproc

; *****************************************************************************
; * One frame delay                                                           *
; *****************************************************************************
.proc frame_delay
:
  lda VCOUNT
  bne :-
  
:
  lda VCOUNT
  beq :-
  
  rts
.endproc

.ifdef Atari5200

; *****************************************************************************
; * Initialise trak-ball reading centre position                              *
; *****************************************************************************
.proc init_trak_ball
  ; disable pots
  lda #$00
  sta CONSOL
    
  jsr frame_delay
  jsr frame_delay

  ; start POT read
  sta POTGO

  jsr frame_delay
  jsr frame_delay

  ; read centre position
  lda POT2
  sta zp_ball_cx
  lda #$00
  sta zp_calibrate
  rts
.endproc

; *****************************************************************************
; * Select which keypad is active depending on contol option - joystick in    *
; * port 1, trakball in port 2                                                *
; *****************************************************************************
.proc select_keypad
  ldx ship_control
  lda consol_values,x
  sta CONSOL
  rts

consol_values:
  .byte $04,$04,$05
.endproc

; *****************************************************************************
; * In-game trakball calibration                                              *
; *****************************************************************************
.proc calibrate_trakball
  dec zp_calibrate
  lda zp_calibrate
  cmp #$03
  beq start_calibration
  cmp #$01
  beq read_pot
  cmp #$00
  beq finish_calibration
  rts

start_calibration:
  ldx ship_control
  lda disable_pot_values,x
  sta CONSOL
  rts

disable_pot_values:
  .byte $00,$00,$01

read_pot:
  sta POTGO
  rts

finish_calibration:
  ; read centre position
  lda POT2
  sta zp_ball_cx
  sta POTGO
  jmp select_keypad
.endproc

.endif

.include "line_data.s"
.include "subs.s"
.include "ship.s"
.include "charges.s"
.include "missiles.s"
.include "collisions.s"
.include "status.s"
.include "audio.s"
.include "ram.s"

code_end:
