REMOVE_CHARGE = $FF

charge_frame_data:
  .byte $30,$70,$F0,$F0,$F0,$F0,$E0,$C0
  .byte $00,$F0,$F0,$F0,$F0,$F0,$F0,$00
  .byte $C0,$E0,$F0,$F0,$F0,$F0,$70,$30
  .byte $00,$F0,$F0,$F0,$F0,$F0,$F0,$00
    
; *****************************************************************************
; * Initialise the depth charges                                              *
; *****************************************************************************
.proc init_charges
  ldx #$00
  lda #$00
loop:
  sta PLAYER0_MEMORY,x
  sta PLAYER1_MEMORY,x
  sta PLAYER2_MEMORY,x
  sta PLAYER3_MEMORY,x
  inx
  bne loop
  
  ldx #$00
  lda #$00
loop2:
  sta charge_y,x
  inx
  cpx #NUM_CHARGES
  bcc loop2

  lda #$00
  sta COLPM0
  sta COLPM1
  sta COLPM2
  sta COLPM3
  
  rts
.endproc

; *****************************************************************************
; * Draw all the depth charges                                                *
; *****************************************************************************
.proc draw_charges
  ldx #$00

loop:
  ldy charge_y,x
  beq next

  lda draw_charge_low,x
  sta zp_dest
  lda draw_charge_high,x
  sta zp_dest+1
  jmp (zp_dest)

next:
  inx
  cpx #NUM_CHARGES
  bne loop
  rts

draw_charge_low:
  .byte <draw_charge_P0,<draw_charge_P1,<draw_charge_P2,<draw_charge_P3

draw_charge_high:
  .byte >draw_charge_P0,>draw_charge_P1,>draw_charge_P2,>draw_charge_P3
.endproc

; *****************************************************************************
; * Draw the player 0 depth charge                                            *
; *****************************************************************************
.proc draw_charge_P0
  lda charge_x,x
  cmp #REMOVE_CHARGE
  bne draw_charge
  jmp erase_charge
  
draw_charge:
  inc charge_frame,x
  lda charge_frame,x
  and #$18
  tax
    
  lda #$00
.repeat 2,i
  sta PLAYER0_MEMORY-2+i,y
.endrepeat

.repeat 8,i
  lda charge_frame_data+i,x
  sta PLAYER0_MEMORY+i,y
.endrepeat  

  ldx #$00
  lda charge_x,x
  sta HPOSP0,x
  jmp draw_charges::next
  
erase_charge:
  lda #$00
.repeat 10,i
  sta PLAYER0_MEMORY-2+i,y
.endrepeat
  
  sta charge_y,x
  sta HPOSP0,x
  jmp draw_charges::next
.endproc

; *****************************************************************************
; * Draw the player 1 depth charge                                            *
; *****************************************************************************
.proc draw_charge_P1
  lda charge_x,x
  cmp #REMOVE_CHARGE
  bne draw_charge
  jmp erase_charge
  
draw_charge:
  inc charge_frame,x
  lda charge_frame,x
  and #$18
  tax
    
  lda #$00
.repeat 2,i
  sta PLAYER1_MEMORY-2+i,y
.endrepeat

.repeat 8,i
  lda charge_frame_data+i,x
  sta PLAYER1_MEMORY+i,y
.endrepeat  

  ldx #$01
  lda charge_x,x
  sta HPOSP0,x
  jmp draw_charges::next
  
erase_charge:
  lda #$00
.repeat 10,i
  sta PLAYER1_MEMORY-2+i,y
.endrepeat
  
  sta charge_y,x
  sta HPOSP0,x
  jmp draw_charges::next
.endproc

; *****************************************************************************
; * Draw the player 2 depth charge                                            *
; *****************************************************************************
.proc draw_charge_P2
  lda charge_x,x
  cmp #REMOVE_CHARGE
  bne draw_charge
  jmp erase_charge
  
draw_charge:
  inc charge_frame,x
  lda charge_frame,x
  and #$18
  tax
    
  lda #$00
.repeat 2,i
  sta PLAYER2_MEMORY-2+i,y
.endrepeat

.repeat 8,i
  lda charge_frame_data+i,x
  sta PLAYER2_MEMORY+i,y
.endrepeat  

  ldx #$02
  lda charge_x,x
  sta HPOSP0,x
  jmp draw_charges::next
  
erase_charge:
  lda #$00
.repeat 10,i
  sta PLAYER2_MEMORY-2+i,y
.endrepeat
  
  sta charge_y,x
  sta HPOSP0,x
  jmp draw_charges::next
.endproc

; *****************************************************************************
; * Draw the player 3 depth charge                                            *
; *****************************************************************************
.proc draw_charge_P3
  lda charge_x,x
  cmp #REMOVE_CHARGE
  bne draw_charge
  jmp erase_charge
  
draw_charge:
  inc charge_frame,x
  lda charge_frame,x
  and #$18
  tax
    
  lda #$00
.repeat 2,i
  sta PLAYER3_MEMORY-2+i,y
.endrepeat

.repeat 8,i
  lda charge_frame_data+i,x
  sta PLAYER3_MEMORY+i,y
.endrepeat  

  ldx #$03
  lda charge_x,x
  sta HPOSP0,x
  jmp draw_charges::next
  
erase_charge:
  lda #$00
.repeat 10,i
  sta PLAYER3_MEMORY-2+i,y
.endrepeat
  
  sta charge_y,x
  sta HPOSP0,x
  jmp draw_charges::next
.endproc

; *****************************************************************************
; * Process all the depth charges                                             *
; *****************************************************************************
.proc process_charges
  ldx #$00

loop:
  lda charge_y,x
  beq next

  clc
  adc #$02
  cmp #$E5
  bcs remove_charge
  sta charge_y,x

next:
  inx
  cpx #NUM_CHARGES
  bcc loop
  rts

remove_charge:
  lda #REMOVE_CHARGE
  sta charge_x,x
  bmi next
.endproc

; *****************************************************************************
; * Drop a depth charge                                                       *
; *****************************************************************************
.proc drop_charge
  ldx #$00
loop:
  lda charge_y,x
  beq fire
  inx
  cpx #NUM_CHARGES
  bne loop
  rts
  
fire:
  lda ship_x
  clc
  adc #14
  sta charge_x,x
  lda #56
  sta charge_y,x
  lda #SFX_DROP_CHARGE
  jsr play_sfx
  rts
.endproc
