; *****************************************************************************
; * Check all collisions                                                      *
; *****************************************************************************
.proc check_collisions
  jsr check_missiles_to_charges
  jsr check_charges_collisions
  jsr check_missiles_to_ship
  lda #$01
  sta HITCLR
  rts
.endproc

; *****************************************************************************
; * Check for collisions between missiles and depth charges                   *
; *****************************************************************************
.proc check_missiles_to_charges
  ldx #$00
  
loop:
  lda M0PL,x
  bne hit
  
next:
  inx
  cpx #NUM_MISSILES
  bcc loop
  rts

hit:
  sta zp_temp
  ldy #$00

hit_loop:
  lsr zp_temp
  bcc hit_next

  lda #REMOVE_CHARGE
  sta charge_x,y

hit_next:
  iny
  cpy #NUM_CHARGES
  bcc hit_loop

  lda #SFX_BLOCKED
  jsr play_sfx

  lda #REMOVE_MISSILE
  sta missile_x,x
  bmi next
.endproc

; *****************************************************************************
; * Check for depth charge collisions                                         *
; *****************************************************************************
.proc check_charges_collisions
  ldx #$00

loop:
  lda P0PF,x
  bne check_hit
        
next:
  inx
  cpx #NUM_CHARGES
  bcc loop
  rts

check_hit:
  lda charge_y,x
  cmp #67
  bcs collision_detected
  jmp next

collision_detected:
  ldy #$00
  
check_y:
  txa
  pha
  
  ldx sub_type,y
  
  lda collision_routine_low,x
  sta zp_dest
  lda collision_routine_high,x
  sta zp_dest+1
  
  pla
  tax
  
  jmp (zp_dest)

next_y:
  iny
  cpy #NUM_SUBS
  bne check_y
  beq next
  
collision_routine_low:
  .byte <sub_collision,<sub_collision
  .byte <no_collision,<no_collision,<no_collision,<no_collision
  .byte <eel_collision,<no_collision
  .byte <mine_collision,<no_collision
  .byte <eel_collision
  
collision_routine_high:
  .byte >sub_collision,>sub_collision
  .byte >no_collision,>no_collision,>no_collision,>no_collision
  .byte >eel_collision,>no_collision
  .byte >mine_collision,>no_collision
  .byte >eel_collision
.endproc

; *****************************************************************************
; * Check for collisions between depth charges and subs                       *
; *****************************************************************************
.proc sub_collision
  lda charge_y,x
  clc
  sbc sub_y,y
  sbc #$07
  adc #$0F
  bcs check_x
  jmp check_charges_collisions::next_y
  
check_x:  
  lda charge_x,x
  clc
  sbc sub_x,y
  sbc #$13
  adc #$17
  bcs hit
  jmp check_charges_collisions::next_y

hit:
  lda #REMOVE_CHARGE
  sta charge_x,x
  jsr explode_sub
  jsr update_score
  jmp check_charges_collisions::next

sub_y:
  .repeat 9,i
  .byte 81+(17*i)
  .endrepeat
.endproc

; *****************************************************************************
; * Null depth charge collision                                               *
; *****************************************************************************
.proc no_collision
  jmp check_charges_collisions::next_y
.endproc

; *****************************************************************************
; * Check for collisions between depth charges and eels                       *
; *****************************************************************************
.proc eel_collision
  lda charge_y,x
  clc
  sbc eel_y,y
  sbc #14
  adc #23
  bcs check_x
  jmp check_charges_collisions::next_y
  
check_x:  
  lda charge_x,x
  clc
  sbc sub_x,y
  sbc #47
  adc #51
  bcs hit
  jmp check_charges_collisions::next_y

hit:
  lda #REMOVE_CHARGE
  sta charge_x,x
  jsr explode_eel
  jsr electrocute_ship
  jmp check_charges_collisions::next

eel_y:
  .repeat 9,i
  .byte 76+(17*i)
  .endrepeat
.endproc

; *****************************************************************************
; * Check for collisions between depth charges and mines                      *
; *****************************************************************************
.proc mine_collision
  lda charge_y,x
  clc
  sbc mine_y,y
  sbc #14
  adc #23
  bcs check_x
  jmp check_charges_collisions::next_y
  
check_x:  
  lda charge_x,x
  clc
  sbc sub_x,y
  sbc #15
  adc #19
  bcs hit
  jmp check_charges_collisions::next_y

hit:
  lda #REMOVE_CHARGE
  sta charge_x,x
  jsr explode_mine

  ldy #$00
destroy_all_subs:
  lda sub_type,y
  cmp #SUB_EXPLODE
  bcs not_a_sub
  jsr explode_sub
  jsr update_score
not_a_sub:
  iny
  cpy #NUM_SUBS
  bcc destroy_all_subs

  lda #$10
  sta mine_detonated
  
  jmp check_charges_collisions::next

mine_y:
  .repeat 9,i
  .byte 76+(17*i)
  .endrepeat
.endproc

; *****************************************************************************
; * Check for collisions between missiles and the ship                        *
; *****************************************************************************
.proc check_missiles_to_ship
  ldx #$00
  
loop:
  lda M0PF,x
  and #$0B
  bne check_y
  
next:
  inx
  cpx #NUM_MISSILES
  bcc loop
  rts

check_y:
  lda missile_y,x
  clc
  sbc #41
  sbc #$17
  adc #$1F
  bcc next
  
hit:
  lda #REMOVE_MISSILE
  sta missile_x,x
  jmp explode_ship
.endproc
