REMOVE_MISSILE = $FF

; *****************************************************************************
; * Initialise missiles                                                       *
; *****************************************************************************
.proc init_missiles
  ldx #$00
  lda #$00
loop:
  sta MISSILE_MEMORY,x
  inx
  bne loop
  
  ldx #$00
  lda #$00
loop2:
  sta missile_y,x
  inx
  cpx #NUM_MISSILES
  bcc loop2

  lda #$06
  sta fire_level

  rts
.endproc

; *****************************************************************************
; * Draw all missiles                                                         *
; *****************************************************************************
.proc draw_missiles
  inc missile_colour
  lda missile_colour
  sta COLPF3

  ldx #$00

loop:
  ldy missile_y,x
  beq next

  lda missile_x,x
  cmp #REMOVE_MISSILE
  bne draw_missile
  jmp remove_missile

draw_missile:
  lda MISSILE_MEMORY,y
  ora or_data,x
  sta MISSILE_MEMORY,y
  lda MISSILE_MEMORY+1,y
  ora or_data,x
  sta MISSILE_MEMORY+1,y
  lda MISSILE_MEMORY+2,y
  ora or_data,x
  sta MISSILE_MEMORY+2,y
  lda MISSILE_MEMORY+3,y
  ora or_data,x
  sta MISSILE_MEMORY+3,y
  lda MISSILE_MEMORY+4,y
  ora or_data,x
  sta MISSILE_MEMORY+4,y
  lda MISSILE_MEMORY+5,y
  ora or_data,x
  sta MISSILE_MEMORY+5,y
  lda MISSILE_MEMORY+6,y
  ora or_data,x
  sta MISSILE_MEMORY+6,y
  lda MISSILE_MEMORY+7,y
  ora or_data,x
  sta MISSILE_MEMORY+7,y

  lda MISSILE_MEMORY+8,y
  and and_data,x
  sta MISSILE_MEMORY+8,y

  lda missile_x,x
  sta HPOSM0,x

next:
  inx
  cpx #NUM_MISSILES
  bne loop
  rts

remove_missile:
.repeat 9,i
  lda MISSILE_MEMORY+i,y
  and and_data,x
  sta MISSILE_MEMORY+i,y
.endrepeat

  lda #$00
  sta missile_y,x
  sta HPOSM0,x
  jmp next

and_data:
  .byte $FC,$F3,$CF,$3F

or_data:
  .byte $03,$0C,$30,$C0
.endproc

; *****************************************************************************
; * Process all missiles                                                      *
; *****************************************************************************
.proc process_missiles
  ldx #$00
loop:
  lda missile_y,x
  beq next

  lda ship_type
  cmp #SHIP_ACTIVE
  bne remove_missile
    
  lda missile_sub_count,x
  bne update_x
  
  lda missile_y,x
  sec
  sbc #$01
  cmp #$08
  bcc remove_missile
  sta missile_y,x
  
next:
  inx
  cpx #NUM_MISSILES
  bcc loop
  rts

update_x:
  dec missile_sub_count,x
  bne :+
  lda #SFX_FIRE_MISSILE
  jsr play_sfx
:
  ldy missile_sub,x
  lda sub_x,y
  clc
  adc #$09
  sta missile_x,x
  jmp next
  
remove_missile:
  lda #REMOVE_MISSILE
  sta missile_x,x
  bmi next
.endproc

; *****************************************************************************
; * Fire a missile, sub index in y                                            *
; *****************************************************************************
.proc fire_missile
  cpy fire_level
  bcs random_fire
  lda #$00
  rts
  
random_fire:
  ldx RANDOM
  cpx #$10
  bcc check_ship
  lda #$00
  rts

check_ship:
  lda ship_type
  cmp #SHIP_ACTIVE
  beq try_to_fire
  rts
  
try_to_fire:
  ldx #$00
loop:
  lda missile_y,x
  beq fire
  inx
  cpx #NUM_MISSILES
  bne loop
  lda #$00
  rts
  
fire:
  lda sub_x,y
  clc
  adc #$09
  sta missile_x,x
  lda sub_y,y
  sta missile_y,x
  tya
  sta missile_sub,x
  lda #$20
  sta missile_sub_count,x
  rts
  
sub_y:
  .repeat 9,i
  .byte 76+(17*i)
  .endrepeat
.endproc
