SHIP_ARRIVING  = 0
SHIP_ACTIVE    = 1
SHIP_EXPLODING = 2
SHIP_ELECTRIC  = 3
SHIP_ARRIVING_ELECTRIC = 4
SHIP_EXPLODING_ELECTRIC = 5
SHIP_ROW_LEFT  = 6
SHIP_ROW_RIGHT = 7

DIGITAL_CONTROL  = 0
ANALOG_CONTROL   = 1
TRAKBALL_CONTROL = 2

; *****************************************************************************
; * Convert ship x coord to LMS offset                                        *
; *****************************************************************************
.proc ship_x_to_offset
  txa
  lsr
  lsr
  sta zp_offset
  lda #68
  sec
  sbc zp_offset
  sta zp_offset
  rts
.endproc

; *****************************************************************************
; * Convert rowboat x coord to LMS offset                                     *
; *****************************************************************************
.proc rowboat_x_to_offset
  txa
  lsr
  lsr
  sta zp_offset
  lda #70
  sec
  sbc zp_offset
  sta zp_offset
  rts
.endproc

; *****************************************************************************
; * Initialise ship                                                           *
; *****************************************************************************
.proc init_ship
  lda #SHIP_ARRIVING
  sta ship_type
  lda #$00
  sta ship_x
  sta ship_frame
  rts
.endproc
  
; *****************************************************************************
; * Draw the ship                                                             *
; *****************************************************************************
.proc draw_ship
  ldy ship_type
  cpy #SHIP_ROW_LEFT
  bcc ship
  jmp draw_rowboat

ship:
  ldx ship_x
  jsr ship_x_to_offset
  clc
  adc top_frames_low,y
  sta dli_ship_top_lms
  lda top_frames_high,y
  sta dli_ship_top_lms+1

  lda zp_offset
  clc
  adc middle_frames_low,y
  sta dli_ship_middile_lms
  lda middle_frames_high,y
  sta dli_ship_middile_lms+1

  lda zp_offset
  clc
  adc bottom_frames_low,y
  sta dli_ship_bottom_lms
  lda bottom_frames_high,y
  sta dli_ship_bottom_lms+1

  txa
  and #$03
  sta hscrol_values
  sta hscrol_values+1

  lda ship_frame
  lsr
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets
  sta dli_charsets+1

  ; set colours
  lda ship_colours
  sta dli_colpf0
  and #$0F
  ora #$80
  sta dli_colpf0+1
  lda ship_colours+1
  sta dli_colpf1
  and #$0F
  ora #$80
  sta dli_colpf1+1
  lda ship_colours+2
  sta dli_colpf2
  and #$0F
  ora #$80
  sta dli_colpf2+1

  rts

top_frames_low:
  .byte <ship_frame_top_line,<ship_frame_top_line,<ship_explode_top_line,<ship_electric_top_line
  .byte <ship_electric_top_line,<ship_explode_top_line
  
top_frames_high:
  .byte >ship_frame_top_line,>ship_frame_top_line,>ship_explode_top_line,>ship_electric_top_line
  .byte >ship_electric_top_line,>ship_explode_top_line

middle_frames_low:
  .byte <ship_frame_middle_line,<ship_frame_middle_line,<ship_explode_middle_line,<ship_electric_middle_line
  .byte <ship_electric_middle_line,<ship_explode_middle_line
  
middle_frames_high:
  .byte >ship_frame_middle_line,>ship_frame_middle_line,>ship_explode_middle_line,>ship_electric_middle_line
  .byte >ship_electric_middle_line,>ship_explode_middle_line

bottom_frames_low:
  .byte <ship_frame_bottom_line,<ship_frame_bottom_line,<ship_explode_bottom_line,<ship_electric_bottom_line
  .byte <ship_electric_bottom_line,<ship_explode_bottom_line
  
bottom_frames_high:
  .byte >ship_frame_bottom_line,>ship_frame_bottom_line,>ship_explode_bottom_line,>ship_electric_bottom_line
  .byte >ship_electric_bottom_line,>ship_explode_bottom_line

charsets:
  .byte >charset1,>charset2,>charset3,>charset4
.endproc

.proc draw_rowboat
  tya
  sec
  sbc #SHIP_ROW_LEFT
  tay

  ldx ship_x
  jsr rowboat_x_to_offset
  clc
  adc #<blank_line
  sta dli_ship_top_lms
  lda #>blank_line
  sta dli_ship_top_lms+1

  lda zp_offset
  clc
  adc top_frames_low,y
  sta dli_ship_middile_lms
  lda top_frames_high,y
  sta dli_ship_middile_lms+1

  lda zp_offset
  clc
  adc bottom_frames_low,y
  sta dli_ship_bottom_lms
  lda bottom_frames_high,y
  sta dli_ship_bottom_lms+1

  txa
  and #$03
  sta hscrol_values
  sta hscrol_values+1

  lda ship_frame
  and #$03
  tax
  lda charsets,x
  sta dli_charsets
  sta dli_charsets+1

  ; set colours
  lda rowboat_colours
  sta dli_colpf0
  and #$0F
  ora #$80
  sta dli_colpf0+1
  lda rowboat_colours+1
  sta dli_colpf1
  and #$0F
  ora #$80
  sta dli_colpf1+1
  lda rowboat_colours+2
  sta dli_colpf2
  and #$0F
  ora #$80
  sta dli_colpf2+1

  rts
  
top_frames_low:
  .byte <rower_left_top_line,<rower_right_top_line

top_frames_high:
  .byte >rower_left_top_line,>rower_right_top_line

bottom_frames_low:
  .byte <rower_left_bottom_line,<rower_right_bottom_line

bottom_frames_high:
  .byte >rower_left_bottom_line,>rower_right_bottom_line

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Process the ship                                                          *
; *****************************************************************************
.proc process_ship
  ldx ship_type
  lda process_routines_low,x
  sta zp_dest
  lda process_routines_high,x
  sta zp_dest+1
  jmp (zp_dest)

process_routines_low:
  .byte <process_ship_arriving,<process_ship_active,<process_ship_exploding,<process_ship_electric
  .byte <process_ship_arriving_electric,<process_ship_exploding_electric
  .byte <process_row_left,<process_row_right
process_routines_high:
  .byte >process_ship_arriving,>process_ship_active,>process_ship_exploding,>process_ship_electric
  .byte >process_ship_arriving_electric,>process_ship_exploding_electric
  .byte >process_row_left,>process_row_right
.endproc

; *****************************************************************************
; * Process an arriving ship                                                  *
; *****************************************************************************
.proc process_ship_arriving
  inc ship_frame
  
  lda ship_x
  clc
  adc #$04
  sta ship_x
  cmp #112
  bcs go_active
  rts

go_active:
  lda #SHIP_ACTIVE
  sta ship_type
  lda zp_trig0
  sta old_trig0
  rts
.endproc

; *****************************************************************************
; * Process an active ship                                                    *
; *****************************************************************************
.proc process_ship_active
  inc ship_frame

  lda game_state
  cmp #GAME_ACTIVE
  beq game_active
  rts

game_active:
  lda ship_control
  cmp #DIGITAL_CONTROL
  bne :+
  jmp digital_control

:
  cmp #ANALOG_CONTROL
  bne :+
  jmp analog_control
  
:
  jmp trakball_control
.endproc

; *****************************************************************************
; * Control ship with digital joystick                                        *
; *****************************************************************************
.proc digital_control
  lda STICK0
  and #$04
  bne try_right
 
  lda ship_x
  sec
  sbc #$02
  cmp #34
  bcs :+
  lda #34
:
  sta ship_x
  jmp try_fire

try_right:
  lda STICK0
  and #$08
  beq move_right
  jmp try_fire

move_right:
  lda ship_x
  clc
  adc #$02
  cmp #190 ; 176
  bcc :+
  lda #190 ; 176
:
  sta ship_x

try_fire:
  lda zp_trig0
  cmp old_trig0
  bne trigger_changed
  rts

trigger_changed:  
  sta old_trig0
  cmp #$00
  beq fire
  rts

fire:
  jmp drop_charge
.endproc

; *****************************************************************************
; * Control ship with analog joystick                                         *
; *****************************************************************************
.proc analog_control
  lda #$00
  sta zp_temp
  
  ldx #(NUM_POT_READINGS-1)
loop:
  clc
  adc pot0_readings,x
  bcc :+
  inc zp_temp
:
  dex
  bpl loop

  ror zp_temp
  ror
  ror zp_temp
  ror
  
.ifndef Atari5200
  sta zp_temp
  lda #$E4
  sec
  sbc zp_temp
.endif

check_left:
  cmp #34
  bcs check_right
  lda #34
  sta ship_x
  jmp digital_control::try_fire

check_right:
  cmp #190
  bcc save_x
  lda #190
  
save_x:
  sta ship_x

try_fire:
  jmp digital_control::try_fire
.endproc

; *****************************************************************************
; * Control ship with trakball in port 2                                      *
; *****************************************************************************
.proc trakball_control
  lda zp_pot2
  sec
  sbc zp_ball_cx

  ; round up by 2
  clc
  adc #$02
  ; divide by 4
  cmp #$80 ; puts sign in C
  php      ; signed shift right twice
  ror
  plp
  ror

  clc
  adc ship_x
  
check_left:
  cmp #34
  bcs check_right
  lda #34
  sta ship_x
  jmp digital_control::try_fire

check_right:
  cmp #190
  bcc save_x
  lda #190
  
save_x:
  sta ship_x

try_fire:
  jmp digital_control::try_fire
.endproc

; *****************************************************************************
; * Process an exploding ship                                                 *
; *****************************************************************************
.proc process_ship_exploding
  inc ship_frame
  lda ship_frame
  cmp #$20
  bcs explosion_complete
  rts

explosion_complete:
  lda game_state
  cmp #GAME_ACTIVE
  bne ship
  
  lda RANDOM
  cmp #$40
  bcc rowboat

ship:  
  lda #SHIP_ARRIVING
  sta ship_type
  lda #$00
  sta ship_x
  rts
  
rowboat:
  lda ship_x
  clc
  adc #$08
  sta ship_x
  cmp #112
  bcs row_left
  lda #SHIP_ROW_RIGHT
  bne save_type
  
row_left:
  lda #SHIP_ROW_LEFT
  
save_type:
  sta ship_type
  rts
.endproc

; *****************************************************************************
; * Process an electric ship                                                  *
; *****************************************************************************
.proc process_ship_electric
  inc ship_frame
  lda ship_frame
  cmp #$80
  bcs electrocution_complete
  and #$08
  bne sfx
  rts

sfx:
  lda #SFX_ELECTRIC
  jsr play_sfx
  rts

electrocution_complete:
  lda #SHIP_ACTIVE
  sta ship_type
  lda zp_trig0
  sta old_trig0
  rts
.endproc

; *****************************************************************************
; * Process an arriving electric ship                                         *
; *****************************************************************************
.proc process_ship_arriving_electric
  inc ship_frame
  lda ship_frame
  cmp #$80
  bcs electrocution_complete
  and #$08
  beq update_x

  lda #SFX_ELECTRIC
  jsr play_sfx

update_x:
  lda ship_x
  clc
  adc #$04
  sta ship_x
  cmp #112
  bcs in_position
  rts

electrocution_complete:
  lda #SHIP_ARRIVING
  sta ship_type
  rts

in_position:
  lda #SHIP_ELECTRIC
  sta ship_type
  rts
.endproc

; *****************************************************************************
; * Process an exploding electric ship                                        *
; *****************************************************************************
.proc process_ship_exploding_electric
  inc ship_frame
  lda ship_frame
  cmp #$20
  bcs explosion_complete
  and #$08
  bne sfx
  rts

sfx:
  lda #SFX_ELECTRIC
  jsr play_sfx
  rts

explosion_complete:
  lda #SHIP_ARRIVING_ELECTRIC
  sta ship_type
  lda #$00
  sta ship_x
  rts
.endproc

; *****************************************************************************
; * Process row left                                                          *
; *****************************************************************************
.proc process_row_left
  lda game_state
  cmp #GAME_ACTIVE
  bne change_to_ship
  
  lda zp_trig0
  cmp old_trig0
  bne trigger_changed

  lda STICK0
  and #$0C
  cmp #$0C
  bne check_stick

  rts

check_stick:
  cmp old_stick0
  bne stick0_changed
  rts
  
stick0_changed:
  sta old_stick0  
  jmp row

trigger_changed:  
  sta old_trig0

row:
  lda #SFX_ROW
  jsr play_sfx

  inc ship_frame
  lda ship_x
  sec
  sbc #$02
  sta ship_x
  cmp #34
  bcc change_to_ship
  rts
  
change_to_ship:
  lda #SHIP_ARRIVING
  sta ship_type
  lda #$00
  sta ship_x
  rts  
.endproc

; *****************************************************************************
; * Process row right                                                         *
; *****************************************************************************
.proc process_row_right
  lda game_state
  cmp #GAME_ACTIVE
  bne change_to_ship
  
  lda zp_trig0
  cmp old_trig0
  bne trigger_changed

  lda STICK0
  and #$0C
  cmp #$0C
  bne check_stick

  rts

check_stick:
  cmp old_stick0
  bne stick0_changed
  rts
  
stick0_changed:
  sta old_stick0  
  jmp row

trigger_changed:  
  sta old_trig0

row:
  lda #SFX_ROW
  jsr play_sfx

  inc ship_frame
  lda ship_x
  clc
  adc #$02
  sta ship_x
  cmp #205
  bcs change_to_ship
  rts
  
change_to_ship:
  lda #SHIP_ARRIVING
  sta ship_type
  lda #$00
  sta ship_x
  rts  
.endproc

; *****************************************************************************
; * Explode the ship                                                          *
; *****************************************************************************
.proc explode_ship
  lda ship_type
  cmp #SHIP_ACTIVE
  beq explode
  rts
  
explode:
  lda #SHIP_EXPLODING
  sta ship_type
  lda #$00
  sta ship_frame
  lda #CALIBRATE_TRAKBALL_REQUEST
  sta zp_calibrate

  lda #SFX_EXPLODE
  jsr play_sfx

  rts
.endproc

; *****************************************************************************
; * Electrocute the ship                                                      *
; *****************************************************************************
.proc electrocute_ship
  lda ship_type
  cmp #SHIP_ARRIVING
  beq electrocute1
  cmp #SHIP_ARRIVING_ELECTRIC
  beq electrocute1
  cmp #SHIP_ACTIVE
  beq electrocute2
  cmp #SHIP_ELECTRIC
  beq electrocute2
  cmp #SHIP_EXPLODING
  beq electrocute3
  rts

electrocute1:
  lda #SHIP_ARRIVING_ELECTRIC
  sta ship_type
  
reset_frame:
  lda #$00
  sta ship_frame
  
  lda #SFX_ELECTRIC
  jsr play_sfx
  rts
  
electrocute2:
  lda #SHIP_ELECTRIC
  sta ship_type
  lda #CALIBRATE_TRAKBALL_REQUEST
  sta zp_calibrate
  jmp reset_frame

electrocute3:
  lda #SHIP_EXPLODING_ELECTRIC
  sta ship_type
  
  lda #SFX_ELECTRIC
  jsr play_sfx
  rts
.endproc
