; *****************************************************************************
; * Initialise the score                                                      *
; *****************************************************************************
.proc init_score
  lda #$00
  sta score
  sta score+1
  sta score+2
  sta old_score
  sta tick
  ldx game_state
  lda times,x
  sta time
  lda PAL
  and #$0F
  cmp #$01
  beq pal_timing
  lda #60
  sta ticks_per_second
  rts

pal_timing:
  lda #50
  sta ticks_per_second
  rts

times:
  .byte $00,$90
.endproc

; *****************************************************************************
; * Update the score, sub index in y                                          *
; *****************************************************************************
.proc update_score
  lda time
  bne active
  rts
  
active:
  sed

  lda score+$02
  clc
  adc increments,y
  sta score+$02
  bcc exit
  
  lda score+$01
  adc #$00
  sta score+$01

  bcc exit

  lda score
  adc #$00
  sta score
  
exit:
  jsr check_extended_play  

  cld
  rts

increments:
  .byte $00,$00,$00,$00,$00,$10,$20,$50,$75
.endproc

; *****************************************************************************
; * Check the score for extended play every 10000 points                      *
; *****************************************************************************
.proc check_extended_play
  lda score+1
  and #$10
  cmp old_score
  bne extended_play
  rts

extended_play:
  sta old_score

  lda time
  clc
  adc #$45
  bcs set_max
  cmp #$90
  bcc save_time
set_max:
  lda #$90
save_time:
  sta time

  lda #$02
  sta text_request

  lda fire_level
  beq exit
  dec fire_level
exit:
  rts

.endproc

; *****************************************************************************
; * Check whether the current score beats the high score                      *
; *****************************************************************************
.proc check_high_score
  ldx #$00

check_loop:
  lda high_score,x
  cmp score,x
  beq check_next
  bcc new_high
  rts

check_next:
  inx
  cpx #$03
  bcc check_loop
  rts

new_high:
  ldx #$00
high_loop:
  lda score,x
  sta high_score,x
  inx
  cpx #$03
  bcc high_loop
  rts
.endproc

; *****************************************************************************
; * Update the count down timer                                               *
; *****************************************************************************
.proc update_time
  lda game_state
  cmp #GAME_ACTIVE
  beq game_active
  rts
  
game_active:
  inc tick
  lda tick
  cmp ticks_per_second
  bcs one_second
  rts

one_second:
  lda #$00
  sta tick
  lda time
  bne decrement_time
  rts

decrement_time:
  sed
  lda time
  sec
  sbc #$01
  sta time
  cld

  lda time
  beq game_over
  rts

game_over:
  lda #TITLE_SCREEN
  sta game_state
  lda #$06
  sta fire_level

  ldy #$01
  lda RANDOM
  and #$01
  jsr new_text
  
  ldy #$08
  jmp set_controller_text
.endproc

; *****************************************************************************
; * Write the score, high score and countdown time to the status display      *
; *****************************************************************************
.proc display_status
; display score
.repeat 3,i
  lda score+i
  lsr
  lsr
  lsr
  lsr
  tax
  lda digits,x
  sta status_line+6+(2*i)
  lda score+i
  and #$0F
  tax
  lda digits,x
  sta status_line+6+(2*i)+1
.endrepeat

  jsr check_high_score

; display high score
.repeat 3,i
  lda high_score+i
  lsr
  lsr
  lsr
  lsr
  tax
  lda digits,x
  sta status_line+21+(2*i)
  lda high_score+i
  and #$0F
  tax
  lda digits,x
  sta status_line+21+(2*i)+1
.endrepeat

  jsr update_time
  
; display time
  lda time
  lsr
  lsr
  lsr
  lsr
  tax
  lda digits,x
  sta status_line+38
  lda time
  and #$0F
  tax
  lda digits,x
  sta status_line+39

  rts
  
digits:
  scrcode "0123456789"
.endproc
