SUB_LEFT            = 0
SUB_RIGHT           = 1
SUB_EXPLODE         = 2
SUB_SCORE           = 3
SUB_TEXT_LEFT       = 4
SUB_TEXT_RIGHT      = 5
SUB_EEL_LEFT        = 6
SUB_EEL_EXPLODE     = 7
SUB_MINE            = 8
SUB_MINE_EXPLODE    = 9
SUB_EEL_RIGHT       = 10

; *****************************************************************************
; * Convert sub x coord to LMS offset                                         *
; *****************************************************************************
.proc sub_x_to_offset
  txa
  lsr
  lsr
  sta zp_offset
  lda #69
  sec
  sbc zp_offset
  sta zp_offset
  rts
.endproc

; *****************************************************************************
; * Convert electric eel x coord to LMS offset                                *
; *****************************************************************************
.proc eel_x_to_offset
  txa
  lsr
  lsr
  sta zp_offset
  lda #66
  sec
  sbc zp_offset
  sta zp_offset
  rts
.endproc

; *****************************************************************************
; * Convert mine x coord to LMS offset                                        *
; *****************************************************************************
.proc mine_x_to_offset
  txa
  lsr
  lsr
  sta zp_offset
  lda #71
  sec
  sbc zp_offset
  sta zp_offset
  rts
.endproc

; *****************************************************************************
; * Convert text x coord to LMS offset                                        *
; *****************************************************************************
.proc text_x_to_offset
  txa
  lsr
  sta zp_offset
  lda #128
  sec
  sbc zp_offset
  sta zp_offset
  rts
.endproc

; *****************************************************************************
; * Initialise a full set of subs                                             *
; *****************************************************************************
.proc init_subs
  ldy #$00
loop:
  jsr new_sub
  iny
  cpy #NUM_SUBS
  bcc loop
  
  lda #$00
  sta text_request
  sta mine_detonated
  
  rts
.endproc

; *****************************************************************************
; * Draw all the subs                                                         *
; *****************************************************************************
.proc draw_subs
  ldy #$00
  
loop:
  ldx  sub_type,y
  
  lda draw_routines_low,x
  sta zp_dest
  lda draw_routines_high,x
  sta zp_dest+1
  jmp (zp_dest)

next:
  iny
  cpy #NUM_SUBS
  bcc loop

  jmp update_dlist

draw_routines_low:
  .byte <draw_left_sub,<draw_right_sub,<draw_explode_sub,<draw_score,<draw_text,<draw_text
  .byte <draw_eel_left,<draw_eel_explode
  .byte <draw_mine,<draw_mine_explode
  .byte <draw_eel_right
draw_routines_high:
  .byte >draw_left_sub,>draw_right_sub,>draw_explode_sub,>draw_score,>draw_text,>draw_text
  .byte >draw_eel_left,>draw_eel_explode
  .byte >draw_mine,>draw_mine_explode
  .byte >draw_eel_right
.endproc

; *****************************************************************************
; * Draw a left sub                                                           *
; *****************************************************************************
.proc draw_left_sub
  ldx sub_x,y  
  jsr sub_x_to_offset
  clc
  adc top_lines_low,y
  sta temp_top_low,y
  lda top_lines_high,y
  sta temp_top_high,y
  
  lda zp_offset
  clc
  adc bottom_lines_low,y
  sta temp_bottom_low,y
  lda bottom_lines_high,y
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

top_lines_low:
  .byte <sub_left_top_line,<sub_left_top_line,<sub_left_top_line,<sub_left_top_line,<sub_left_top_line
  .byte <sub2_left_top_line,<sub2_left_top_line,<sub2_left_top_line,<sub2_left_top_line

top_lines_high:
  .byte >sub_left_top_line,>sub_left_top_line,>sub_left_top_line,>sub_left_top_line,>sub_left_top_line
  .byte >sub2_left_top_line,>sub2_left_top_line,>sub2_left_top_line,>sub2_left_top_line

bottom_lines_low:
  .byte <sub_left_bottom_line,<sub_left_bottom_line,<sub_left_bottom_line,<sub_left_bottom_line,<sub_left_bottom_line
  .byte <sub2_left_bottom_line,<sub2_left_bottom_line,<sub2_left_bottom_line,<sub2_left_bottom_line

bottom_lines_high:
  .byte >sub_left_bottom_line,>sub_left_bottom_line,>sub_left_bottom_line,>sub_left_bottom_line,>sub_left_bottom_line
  .byte >sub2_left_bottom_line,>sub2_left_bottom_line,>sub2_left_bottom_line,>sub2_left_bottom_line

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Draw a right sub                                                          *
; *****************************************************************************
.proc draw_right_sub
  ldx sub_x,y  
  jsr sub_x_to_offset
  clc
  adc top_lines_low,y
  sta temp_top_low,y
  lda top_lines_high,y
  sta temp_top_high,y
  
  lda zp_offset
  clc
  adc bottom_lines_low,y
  sta temp_bottom_low,y
  lda bottom_lines_high,y
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

top_lines_low:
  .byte <sub_right_top_line,<sub_right_top_line,<sub_right_top_line,<sub_right_top_line,<sub_right_top_line
  .byte <sub2_right_top_line,<sub2_right_top_line,<sub2_right_top_line,<sub2_right_top_line

top_lines_high:
  .byte >sub_right_top_line,>sub_right_top_line,>sub_right_top_line,>sub_right_top_line,>sub_right_top_line
  .byte >sub2_right_top_line,>sub2_right_top_line,>sub2_right_top_line,>sub2_right_top_line

bottom_lines_low:
  .byte <sub_right_bottom_line,<sub_right_bottom_line,<sub_right_bottom_line,<sub_right_bottom_line,<sub_right_bottom_line
  .byte <sub2_right_bottom_line,<sub2_right_bottom_line,<sub2_right_bottom_line,<sub2_right_bottom_line

bottom_lines_high:
  .byte >sub_right_bottom_line,>sub_right_bottom_line,>sub_right_bottom_line,>sub_right_bottom_line,>sub_right_bottom_line
  .byte >sub2_right_bottom_line,>sub2_right_bottom_line,>sub2_right_bottom_line,>sub2_right_bottom_line

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Draw an exploding sub                                                     *
; *****************************************************************************
.proc draw_explode_sub
  ldx sub_x,y  
  jsr sub_x_to_offset
  clc
  adc #<sub_explode_top_line
  sta temp_top_low,y
  lda #>sub_explode_top_line
  sta temp_top_high,y
  
  lda zp_offset
  clc
  adc #<sub_explode_bottom_line
  sta temp_bottom_low,y
  lda #>sub_explode_bottom_line
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Draw a score                                                              *
; *****************************************************************************
.proc draw_score
  ldx sub_x,y
  jsr sub_x_to_offset
  clc
  adc score_top_low,y
  sta temp_top_low,y
  lda score_top_high,y
  sta temp_top_high,y

  lda zp_offset
  clc
  adc score_bottom_low,y
  sta temp_bottom_low,y
  lda score_bottom_high,y
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda #>digits_charset
  sta dli_charsets+2,y

  jmp draw_subs::next

score_top_low:
  .byte 0,0,0,0,0
  .byte <score_100_top_line,<score_200_top_line,<score_500_top_line,<score_750_top_line
score_top_high:
  .byte 0,0,0,0,0
  .byte >score_100_top_line,>score_200_top_line,>score_500_top_line,>score_750_top_line
score_bottom_low:
  .byte 0,0,0,0,0
  .byte <score_100_bottom_line,<score_200_bottom_line,<score_500_bottom_line,<score_750_bottom_line
score_bottom_high:
  .byte 0,0,0,0,0
  .byte >score_100_bottom_line,>score_200_bottom_line,>score_500_bottom_line,>score_750_bottom_line
.endproc

; *****************************************************************************
; * Draw text                                                                 *
; *****************************************************************************
.proc draw_text
  ldx sub_x,y
  jsr text_x_to_offset

  txa
  and #$01
  asl
  sta hscrol_values+2,y
  
  ldx sub_frame,y
  lda zp_offset
  clc
  adc text_top_low,x
  sta temp_top_low,y
  lda text_top_high,x
  sta temp_top_high,y

  lda zp_offset
  clc
  adc text_bottom_low,x
  sta temp_bottom_low,y
  lda text_bottom_high,x
  sta temp_bottom_high,y

  lda #>digits_charset
  sta dli_charsets+2,y

  jmp draw_subs::next

text_top_low:
  .byte <blowsub_top_line,<press_start_top_line,<extended_play_top_line
  .byte <digital_control_top_line,<analog_control_top_line,<trakball_control_top_line
text_top_high:
  .byte >blowsub_top_line,>press_start_top_line,>extended_play_top_line
  .byte >digital_control_top_line,>analog_control_top_line,>trakball_control_top_line
text_bottom_low:
  .byte <blowsub_bottom_line,<press_start_bottom_line,<extended_play_bottom_line
  .byte <digital_control_bottom_line,<analog_control_bottom_line,<trakball_control_bottom_line
text_bottom_high:
  .byte >blowsub_bottom_line,>press_start_bottom_line,>extended_play_bottom_line
  .byte >digital_control_bottom_line,>analog_control_bottom_line,>trakball_control_bottom_line
.endproc

; *****************************************************************************
; * Draw a left eel                                                           *
; *****************************************************************************
.proc draw_eel_left
  ldx sub_x,y
  jsr eel_x_to_offset
  clc
  adc #<eel_left_top_line
  sta temp_top_low,y
  lda #>eel_left_top_line
  sta temp_top_high,y

  lda zp_offset
  clc
  adc #<eel_left_bottom_line
  sta temp_bottom_low,y
  lda #>eel_left_bottom_line
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

charsets:
  .byte >charset1,>charset2,>charset3,>charset4
.endproc

; *****************************************************************************
; * Draw a right eel                                                          *
; *****************************************************************************
.proc draw_eel_right
  ldx sub_x,y
  jsr eel_x_to_offset
  clc
  adc #<eel_right_top_line
  sta temp_top_low,y
  lda #>eel_right_top_line
  sta temp_top_high,y

  lda zp_offset
  clc
  adc #<eel_right_bottom_line
  sta temp_bottom_low,y
  lda #>eel_right_bottom_line
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Draw an exploding eel                                                     *
; *****************************************************************************
.proc draw_eel_explode
  ldx sub_x,y
  jsr eel_x_to_offset
  clc
  adc #<eel_explode_top_line
  sta temp_top_low,y
  lda #>eel_explode_top_line
  sta temp_top_high,y

  lda zp_offset
  clc
  adc #<eel_explode_bottom_line
  sta temp_bottom_low,y
  lda #>eel_explode_bottom_line
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

charsets:
  .byte >charset1,>charset2,>charset3,>charset4
.endproc

; *****************************************************************************
; * Draw a mine                                                               *
; *****************************************************************************
.proc draw_mine
  ldx sub_x,y
  jsr mine_x_to_offset
  clc
  adc #<mine_frame_top_line
  sta temp_top_low,y
  lda #>mine_frame_top_line
  sta temp_top_high,y

  lda zp_offset
  clc
  adc #<mine_frame_bottom_line
  sta temp_bottom_low,y
  lda #>mine_frame_bottom_line
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Draw an exploding mine                                                    *
; *****************************************************************************
.proc draw_mine_explode
  ldx sub_x,y
  jsr mine_x_to_offset
  clc
  adc #<mine_explode_top_line
  sta temp_top_low,y
  lda #>mine_explode_top_line
  sta temp_top_high,y

  lda zp_offset
  clc
  adc #<mine_explode_bottom_line
  sta temp_bottom_low,y
  lda #>mine_explode_bottom_line
  sta temp_bottom_high,y

  txa
  and #$03
  sta hscrol_values+2,y

  lda sub_frame,y
  lsr
  lsr
  and #$03
  tax
  lda charsets,x
  sta dli_charsets+2,y

  jmp draw_subs::next

charsets:
  .byte >charset5,>charset6,>charset7,>charset8
.endproc

; *****************************************************************************
; * Set all the LMS addresses in the display list                             *
; *****************************************************************************
.proc update_dlist
  ldx #$00
  ldy #$00

loop:
  lda temp_top_low,x
  sta dli_subs_lms+2,y
  lda temp_top_high,x
  sta dli_subs_lms+3,y
  lda temp_bottom_low,x
  sta dli_subs_lms+5,y
  lda temp_bottom_high,x
  sta dli_subs_lms+6,y
  
  tya
  clc
  adc #$07
  tay

  inx
  cpx #NUM_SUBS
  bne loop

  rts
.endproc

; *****************************************************************************
; * Process all the subs                                                      *
; *****************************************************************************
.proc process_subs
  ldy #$00
  
loop:
  ldx  sub_type,y
  
  lda process_routines_low,x
  sta zp_dest
  lda process_routines_high,x
  sta zp_dest+1
  jmp (zp_dest)

next:
  iny
  cpy #NUM_SUBS
  bcc loop

  rts

process_routines_low:
  .byte <process_left_sub,<process_right_sub,<process_explode_sub,<process_score,<process_text_left,<process_text_right
  .byte <process_eel_left,<process_eel_explode
  .byte <process_mine,<process_mine_explode
  .byte <process_eel_right
process_routines_high:
  .byte >process_left_sub,>process_right_sub,>process_explode_sub,>process_score,>process_text_left,>process_text_right
  .byte >process_eel_left,>process_eel_explode
  .byte >process_mine,>process_mine_explode
  .byte >process_eel_right
.endproc

; *****************************************************************************
; * Process a left sub                                                        *
; *****************************************************************************
.proc process_left_sub
  lda sub_frame,y
  clc
  adc #$01
  sta sub_frame,y
  
  lda sub_fire_delay,y
  beq fire
  sec
  sbc #$01
  sta sub_fire_delay,y
  jmp no_fire

fire:
  jsr fire_missile
  cmp #$00
  beq :+
  clc
  adc #$10
:
  sta sub_fire_delay,y
  
no_fire:
  lda sub_x_low,y
  sec
  sbc sub_dx_low,y
  sta sub_x_low,y
  
  lda sub_x,y
  sbc sub_dx,y
  sta sub_x,y
  
  cmp #$1C
  bcc new
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process a right sub                                                       *
; *****************************************************************************
.proc process_right_sub
  lda sub_frame,y
  clc
  adc #$01
  sta sub_frame,y

  lda sub_fire_delay,y
  beq fire
  sec
  sbc #$01
  sta sub_fire_delay,y
  jmp no_fire

fire:
  jsr fire_missile
  cmp #$00
  beq :+
  clc
  adc #$10
:
  sta sub_fire_delay,y
  
no_fire:
  lda sub_x_low,y
  clc
  adc sub_dx_low,y
  sta sub_x_low,y
  
  lda sub_x,y
  adc sub_dx,y
  sta sub_x,y
  
  cmp #$D0
  bcs new
  jmp process_subs::next
  
new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process an exploding sub                                                  *
; *****************************************************************************
.proc process_explode_sub
  lda sub_frame,y
  clc
  adc #$01
  cmp #$10
  bcs new
  sta sub_frame,y
  jmp process_subs::next

new:
  cpy #$05
  bcs deep
  
shallow:
  jsr new_sub
  jmp process_subs::next

deep:
  lda #SUB_SCORE
  sta sub_type,y
  lda #$00
  sta sub_frame,y
  lda #SFX_POINTS_DING
  jsr play_sfx
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process a score                                                           *
; *****************************************************************************
.proc process_score
  lda sub_frame,y
  clc
  adc #$01
  cmp #$0C
  bcs new
  sta sub_frame,y
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process text left                                                         *
; *****************************************************************************
.proc process_text_left
  lda sub_x,y
  sec
  sbc sub_dx,y
  sta sub_x,y
  cmp #$58
  bcc new
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process text right                                                        *
; *****************************************************************************
.proc process_text_right
  lda sub_x,y
  clc
  adc sub_dx,y
  sta sub_x,y
  cmp #$FE
  bcs new
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process a left eel                                                        *
; *****************************************************************************
.proc process_eel_left
  lda sub_frame,y
  clc
  adc #$01
  sta sub_frame,y
  
  lda sub_x_low,y
  sec
  sbc sub_dx_low,y
  sta sub_x_low,y
  
  lda sub_x,y
  sbc sub_dx,y
  sta sub_x,y
  
  cmp #$02
  bcc new
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process a right eel                                                       *
; *****************************************************************************
.proc process_eel_right
  lda sub_frame,y
  clc
  adc #$01
  sta sub_frame,y
  
  lda sub_x_low,y
  clc
  adc sub_dx_low,y
  sta sub_x_low,y
  
  lda sub_x,y
  adc sub_dx,y
  sta sub_x,y
  
  cmp #216
  bcs new
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process an exploding eel                                                  *
; *****************************************************************************
.proc process_eel_explode
  ldx #$00
  lda ship_type
  cmp #SHIP_ROW_LEFT
  bcs :+
  inx
:

  lda sub_frame,y
  clc
  adc #$01
  cmp limits,x
  bcs new
  sta sub_frame,y
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next

limits:
  .byte $10,$80
.endproc

; *****************************************************************************
; * Process a mine                                                            *
; *****************************************************************************
.proc process_mine
  lda sub_frame,y
  clc
  adc #$01
  sta sub_frame,y
  
  lda sub_x_low,y
  sec
  sbc sub_dx_low,y
  sta sub_x_low,y
  
  lda sub_x,y
  sbc sub_dx,y
  sta sub_x,y
  
  cmp #40
  bcc new
  cmp #209
  bcs new
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Process an exploding mine                                                 *
; *****************************************************************************
.proc process_mine_explode
  lda sub_frame,y
  clc
  adc #$01
  cmp #$10
  bcs new
  sta sub_frame,y
  jmp process_subs::next

new:
  jsr new_sub
  jmp process_subs::next
.endproc

; *****************************************************************************
; * Create a new sub, index in y                                              *
; *****************************************************************************
.proc new_sub
  lda text_request
  beq check_state
  
  ldx #$00
  stx text_request
  jmp new_text
  
check_state:
  lda game_state
  cmp #GAME_ACTIVE
  beq active

  cpy #$01
  beq title_text
  cpy #$08
  beq controller_text
  bne active

title_text:
  lda RANDOM
  and #$01
  jmp new_text

controller_text:
  jmp set_controller_text

active:
  lda RANDOM
  cmp #$E0
  bcc no_eel
  lda RANDOM
  ldx fire_level
  cmp eel_threshold,x
  bcs no_eel
  jmp new_eel

no_eel:
  lda RANDOM
  cmp #$E0
  bcc no_mine
  lda RANDOM
  ldx fire_level
  cmp mine_threshold,x
  bcs no_mine
  jmp new_mine

no_mine:
  lda RANDOM
  and #$1F
  tax

  lda type,x
  sta sub_type,y
  
  lda speed,x
  sta sub_dx,y
  
  lda speed_low,x
  sta sub_dx_low,y

  lda start,x
  sta sub_x,y

  lda #$00
  sta sub_x_low,y

  lda RANDOM
  sta sub_frame,y

  lda #$00
  sta sub_fire_delay,y

  lda RANDOM
  and #$F0
  tax
  ora sub_colours
  sta dli_colpf0+2,y  
  txa
  ora sub_colours+1
  sta dli_colpf1+2,y  
  txa
  ora sub_colours+2
  sta dli_colpf2+2,y  

  rts

type:      .byte SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT
           .byte SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT,SUB_LEFT
           .byte SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT
           .byte SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT,SUB_RIGHT
speed:     .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01
           .byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01
speed_low: .byte $10,$20,$30,$40,$50,$60,$70,$80,$90,$A0,$B0,$C0,$D0,$E0,$F0,$00
           .byte $10,$20,$30,$40,$50,$60,$70,$80,$90,$A0,$B0,$C0,$D0,$E0,$F0,$00

start:     .byte $D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0,$D0
           .byte $1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C,$1C

eel_threshold:
  .byte $80,$70,$60,$50,$40,$30,$20
  
mine_threshold:
  .byte $30,$40,$50,$60,$70,$80,$90

.endproc

; *****************************************************************************
; * Create a new text, frame in a, index in y                                 *
; *****************************************************************************
.proc new_text
  sta sub_frame,y
  
  lda RANDOM
  and #$01
  tax

  lda type,x
  sta sub_type,y
  
  lda speed,x
  sta sub_dx,y

  lda start,x
  sta sub_x,y

  lda RANDOM
  and #$F0
  tax
  ora text_colours
  sta dli_colpf0+2,y  
  txa
  ora text_colours+1
  sta dli_colpf1+2,y  
  txa
  ora text_colours+2
  sta dli_colpf2+2,y  
  rts

type:  .byte SUB_TEXT_LEFT,SUB_TEXT_RIGHT
speed: .byte 1,1
start: .byte $FF,$58
.endproc

.proc set_controller_text
  lda ship_control
  tax
  clc
  adc #$03
  sta sub_frame,y
  
  lda #SUB_TEXT_LEFT
  sta sub_type,y
  
  lda #$00
  sta sub_dx,y

  lda #$A0
  sta sub_x,y

  lda text_colours
  ora #$80
  sta dli_colpf0+2,y  
  lda text_colours+1
  ora #$80
  sta dli_colpf1+2,y
  lda text_colours+2
  ora #$80
  sta dli_colpf2+2,y  
  rts
.endproc

; *****************************************************************************
; * Create a new eel, index in y                                              *
; *****************************************************************************
.proc new_eel
  lda RANDOM
  and #$0F
  tax

  lda type,x
  sta sub_type,y
  
  lda speed,x
  sta sub_dx,y
  
  lda speed_low,x
  sta sub_dx_low,y

  lda start,x
  sta sub_x,y

  lda #$00
  sta sub_x_low,y

  lda RANDOM
  sta sub_frame,y

  lda eel_colours
  sta dli_colpf0+2,y  
  lda eel_colours+1
  sta dli_colpf1+2,y  
  lda eel_colours+2
  sta dli_colpf2+2,y  

  rts

type:      .repeat 8
           .byte SUB_EEL_LEFT
           .endrepeat
           .repeat 8
           .byte SUB_EEL_RIGHT
           .endrepeat
           
speed:     .byte $00,$00,$00,$01,$01,$01,$01,$02
           .byte $00,$00,$00,$01,$01,$01,$01,$02

speed_low: .byte $60,$80,$C0,$00,$40,$80,$C0,$00
           .byte $60,$80,$C0,$00,$40,$80,$C0,$00

start:     .repeat 8
           .byte 216
           .endrepeat
           .repeat 8
           .byte 0
           .endrepeat
           
.endproc

; *****************************************************************************
; * Create a new mine, index in y                                             *
; *****************************************************************************
.proc new_mine
  lda RANDOM
  and #$0F
  tax

  lda #SUB_MINE
  sta sub_type,y
  
  lda speed,x
  sta sub_dx,y
  
  lda speed_low,x
  sta sub_dx_low,y

  lda start,x
  sta sub_x,y

  lda #$00
  sta sub_x_low,y

  lda RANDOM
  sta sub_frame,y

  lda mine_colours
  sta dli_colpf0+2,y  
  lda mine_colours+1
  sta dli_colpf1+2,y  
  lda mine_colours+2
  sta dli_colpf2+2,y  

  rts

speed:     .byte $00,$00,$00,$00,$00,$00,$00,$01
           .byte $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FE
speed_low: .byte $50,$60,$70,$80,$A0,$C0,$E0,$00
           .byte $B0,$A0,$90,$80,$60,$40,$20,$00
start:     .byte 208,208,208,208,208,208,208,208
           .byte 40,40,40,40,40,40,40,40

.endproc

; *****************************************************************************
; * Explode a sub, index in y                                                 *
; *****************************************************************************
.proc explode_sub
  lda #SUB_EXPLODE
  sta sub_type,y
  lda #$00
  sta sub_frame,y
  lda #SFX_EXPLODE
  jsr play_sfx
  rts
.endproc

; *****************************************************************************
; * Explode a eel, index in y                                                 *
; *****************************************************************************
.proc explode_eel
  lda #SUB_EEL_EXPLODE
  sta sub_type,y
  lda #$00
  sta sub_frame,y
  rts
.endproc

; *****************************************************************************
; * Explode a mine, index in y                                                *
; *****************************************************************************
.proc explode_mine
  lda #SUB_MINE_EXPLODE
  sta sub_type,y
  lda #$00
  sta sub_frame,y
  lda #SFX_EXPLODE
  jsr play_sfx
  rts
.endproc
